/* Name:		vokabel.cpp */
/* Autor:		Dirk B"achle */
/* Datum:		11.09.2000 */
/* Zweck:		Definition der Funktionen fr die */
/*			Klassen "vokabel" und "string" */

/** Definition der Funktionen f"ur die Klassen "'vokabel"' und "'string"'.
\file vokabel.cpp
\author Dirk B"achle
\version 1.3
*/

/* ------------------------------------------------ Include-Dateien */

#include <stdio.h>
#include <string.h>
#include "globald.h"
#include "eingabe.h"
#include "vokabel.h"


/* ---------------------------------------------- Globale Variablen */


/* ----------------------------------------------------- Funktionen */


/********************** Klasse stringpiece **************************/

/** Konstruktor der Klasse stringpiece.
@return keiner
*/
stringpiece::stringpiece(void)
{
  this->spiece = (char *) new char[STRINGPIECELEN];

  if (this->spiece == NULL)
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("stringpiece::stringpiece : Konnte keinen Speicher fr spiece allozieren!\n");
    DisplayFreeMemoryMessage();
  }

  this->vorher = NULL;
  this->naechster = NULL;
}

/** Destruktor der Klasse stringpiece.
@return keiner
*/
stringpiece::~stringpiece(void)
{
  delete(this->spiece);
}

/* Setzen von Daten */

/** Setzt den Inhalt des St"uckes auf den "ubergebenen String.
@param piece String-St"uck
@return keiner
*/
void stringpiece::setzen(char *piece)
{
  int i=0;

  while ((i<STRINGPIECELEN) && (piece[i] != 0))
  {
    this->spiece[i] = piece[i];
    i++;
  }

  if (i < STRINGPIECELEN)
  {
    this->spiece[i] = 0;
  }
}

/** Setzt den Vorher-Zeiger auf den "ubergebenen Wert.
@param zeiger Neuer Zeiger
@return keiner
*/
void stringpiece::vorher_setzen(class stringpiece *zeiger)
{
  this->vorher = zeiger;
}

/** Setzt den Naechster-Zeiger auf den "ubergebenen Wert.
@param zeiger Neuer Zeiger
@return keiner
*/
void stringpiece::naechster_setzen(class stringpiece *zeiger)
{
  this->naechster = zeiger;
}



/* Zurckliefern von Daten */

/** Liefert den Inhalt des String-St"uckes zur"uck.
@param piece Zeiger auf den neuen String
@return keiner
*/
void stringpiece::holen(char *piece)
{
  int i=0;

  while ((i < STRINGPIECELEN) && (this->spiece[i] != 0))
  {
    piece[i] = this->spiece[i];
    i++;
  }

  if (i < STRINGPIECELEN)
  {
    piece[i] = 0;
  }
}

/** Liefert den Vorher-Zeiger zur"uck.
@return Vorher-Zeiger
*/
class stringpiece * stringpiece::vorher_holen(void)
{
  return(this->vorher);
}

/** Liefert den Naechster-Zeiger zur"uck.
@return Naechster-Zeiger
*/
class stringpiece * stringpiece::naechster_holen(void)
{
  return(this->naechster);
}


/********************** Klasse string ***************************/

/** Konstruktor der Klasse "'string"'.
@return keiner
*/
string::string()
{
  /* Initialisieren der Variablen */
  this->anfang = NULL;
  this->ende = NULL;
  this->aktuell = NULL;
}

/** Destruktor der Klasse "'string"'.
@return keiner
*/
string::~string()
{
  this->alle_loeschen();
}

/** F"ugt ein neues String-St"uck an der aktuellen Position in die Liste ein.
@return keiner
*/
void string::einfuegen(void)
{
  stringpiece *neu, *tmp;

  neu = (class stringpiece *) new stringpiece;

  if (neu != NULL)
  {
    /* Ist "uberhaupt schon ein Element eingef"ugt? */
    if (this->anfang == NULL)
    {
      /* Nein */
      neu->vorher_setzen(NULL);
      neu->naechster_setzen(NULL);
      this->aktuell = neu;
      this->anfang = neu;
      this->ende = neu;
    }
    else
    {
      /* Einfuegen am Anfang der Liste? */
      if (this->aktuell != this->ende)
      {
	/* Ja, wir sind mitten in der Liste */
	neu->vorher_setzen(this->aktuell);
	tmp = this->aktuell->naechster_holen();
	tmp->vorher_setzen(neu);
	neu->naechster_setzen(tmp);
	this->aktuell->naechster_setzen(neu);
	this->aktuell = neu;

      }
      else
      {
	/* Nein, wir haengen den Eintrag an */
	neu->vorher_setzen(this->aktuell);
	neu->naechster_setzen(NULL);
	this->aktuell->naechster_setzen(neu);
	this->aktuell = neu;
	this->ende = neu;
      }
    }
  }
  else
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("string::einfuegen : Konnte keinen Speicher fr neu allozieren!\n");
    DisplayFreeMemoryMessage();
  }

}

/** L"oscht das aktuelle String-St"uck aus der Liste.
@return keiner
*/
void string::loeschen(void)
{
  stringpiece *tmp1, *tmp2;

  /* Gibt es "uberhaupt etwas zu l"oschen? */
  if (this->anfang != NULL)
  {
    /* Ja, wir k"onnen den aktuellen Eintrag l"oschen. */

    /* Wo befinden wir uns in der Liste? */
    if (this->aktuell == this->anfang)
    {
      /* Wir sind am Anfang der Liste */

      /* Ist dies der einzige Eintrag in der Liste? */
      if (this->aktuell == this->ende)
      {
	/* Ja, es ist der einzige Eintrag. */
	delete this->aktuell;
	this->aktuell = NULL;
	this->anfang = NULL;
	this->ende = NULL;
      }
      else
      {
	/* Nein, es sind noch mehr als ein Eintrag vorhanden. */

	this->anfang = this->aktuell->naechster_holen();
	this->anfang->vorher_setzen(NULL);
	delete this->aktuell;
	this->aktuell = this->anfang;
      }
    }
    else
    {
      if (this->aktuell == this->ende)
      {
	/* Wir sind am Ende der Liste, es sind mehr als */
	/* ein Eintrag vorhanden. */
	tmp1 = this->aktuell->vorher_holen();
	delete this->aktuell;
	this->aktuell = tmp1;
	this->ende = tmp1;
	this->aktuell->naechster_setzen(NULL);
      }
      else
      {
	/* Wir sind mitten in der Liste */
	tmp1 = this->aktuell->vorher_holen();
	tmp2 = this->aktuell->naechster_holen();
	tmp1->naechster_setzen(tmp2);
	tmp2->vorher_setzen(tmp1);
	delete this->aktuell;
	this->aktuell = tmp2;
      }
    }
  }
}

/** L"oscht alle String-St"ucke aus der Liste.
@return keiner
*/
void string::alle_loeschen(void)
{

  while (this->anfang != NULL)
  {
    this->loeschen();
  }
}


/** Bewegt den Zeiger in der Liste einen Schritt vor, falls m"oglich.
@return keiner
*/
void string::vor(void)
{
  if ((this->aktuell != NULL) && (this->aktuell->naechster_holen() != NULL))
  {
    this->aktuell = this->aktuell->naechster_holen();
  }
}

/** Bewegt den Zeiger in der Liste einen Schritt zur"uck, falls m"oglich.
@return keiner
*/
void string::zurueck(void)
{
  if ((this->aktuell != NULL) && (this->aktuell->vorher_holen() != NULL))
  {
    this->aktuell = this->aktuell->vorher_holen();
  }
}

/** Bewegt den Zeiger an den Anfang der Liste.
@return keiner
*/
void string::zum_anfang(void)
{
    this->aktuell = this->anfang;
}

/** Bewegt den Zeiger an das Ende der Liste.
@return keiner
*/
void string::zum_ende(void)
{
    this->aktuell = this->ende;
}

/** Setzt den Gesamt-String auf den "ubergebenen String.
@param str Neuer String
@return keiner
*/
void string::setzen(char *str)
{
  int slen=strlen(str)+1;
  int cnt=0, i=0;
  char pce[STRINGPIECELEN];

  /* Alle bisherigen Teilstuecke loeschen */
  this->alle_loeschen();

  if (str[0] != 0)
  {
    /* Entsprechend viele neue Teilstuecke einfuegen */
    i = (slen/STRINGPIECELEN);
    if ((slen % STRINGPIECELEN) != 0) i++;

    for (cnt = 0; cnt < i; cnt++)
    {
      this->einfuegen();
    }

    /* Auf den Anfang setzen */
    cnt = 0;
    i = 0;
    this->zum_anfang();

    while (cnt<slen)
    {
      while ((i<STRINGPIECELEN) && (str[cnt] != 0))
      {
	pce[i] = str[cnt];
	cnt++;
	i++;
      }

      if (i<STRINGPIECELEN)
      {
	pce[i] = 0;
	cnt++;
      }

      i = 0;
      this->aktuell->setzen(pce);
      this->vor();
    }
  }
}

/** Ermittelt den Gesamt-String.
@param str Neuer String
@return keiner
*/
void string::holen(char *str)
{
  int cnt = 0, i = 0, end = 0;
  char pce[STRINGPIECELEN];

  this->zum_anfang();

  if (this->anfang != NULL)
  {
    while ((this->aktuell != this->ende) && (end == 0))
    {
      this->aktuell->holen(pce);

      while ((pce[i] != 0) && (i < STRINGPIECELEN))
      {
	str[cnt] = pce[i];
	cnt++;
	i++;
      }

      if (i < STRINGPIECELEN)
      {
	str[cnt] = 0;
	end = 1;
      }

      i = 0;
      this->vor();
    }

    if (end == 0)
    {
      this->aktuell->holen(pce);

      while ((pce[i] != 0) && (i < STRINGPIECELEN))
      {
	str[cnt] = pce[i];
	cnt++;
	i++;
      }

      if (i < STRINGPIECELEN)
      {
	str[cnt] = 0;
      }
    }
  }
  else
  {
    str[0] = 0;
  }
}

/********************************************************************/

/********************** Klasse vokabel ******************************/

/** Konstruktor der Klasse vokabel.
@return keiner
*/
vokabel::vokabel(void)
{
  /* Initialisieren der Variablen */
  this->nv = (class string *) new string;
  if (this->nv == NULL)
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("vokabel::vokabel : Konnte keinen Speicher fr nv allozieren!\n");
    DisplayFreeMemoryMessage();
  }

  this->nr = (class string *) new string;
  if (this->nr == NULL)
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("vokabel::vokabel : Konnte keinen Speicher fr nr allozieren!\n");
    DisplayFreeMemoryMessage();
  }

  this->fv = (class string *) new string;
  if (this->fv == NULL)
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("vokabel::vokabel : Konnte keinen Speicher fr fv allozieren!\n");
    DisplayFreeMemoryMessage();
  }

  this->fr = (class string *) new string;
  if (this->fr == NULL)
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("vokabel::vokabel : Konnte keinen Speicher fr fr allozieren!\n");
    DisplayFreeMemoryMessage();
  }

  this->tag = 0;
  this->prozent = 0.0;

  this->vorher = NULL;
  this->naechster = NULL;

}

/** Destruktor der Klasse vokabel.
@return keiner
*/
vokabel::~vokabel(void)
{
  delete(this->nv);
  delete(this->nr);
  delete(this->fv);
  delete(this->fr);
}

/* Setzen von Daten */

/** Setzt die Muttersprachen-Vokabel auf den "ubergebenen String.
@param str Neuer String
@return keiner
*/
void vokabel::nv_setzen(char *str)
{
  this->nv->setzen(str);
}

/** Setzt die Muttersprachen-Bemerkung auf den "ubergebenen String.
@param str Neuer String
@return keiner
*/
void vokabel::nr_setzen(char *str)
{
  this->nr->setzen(str);
}

/** Setzt die Fremdsprachen-Vokabel auf den "ubergebenen String.
@param str Neuer String
@return keiner
*/
void vokabel::fv_setzen(char *str)
{
  this->fv->setzen(str);
}

/** Setzt die Fremdsprachen-Bemerkung auf den "ubergebenen String.
@param str Neuer String
@return keiner
*/
void vokabel::fr_setzen(char *str)
{
  this->fr->setzen(str);
}

/** Setzt die Flag-Variable (gepr"uft/nicht gepr"uft) auf den "ubergebenen Wert.
@param nflag Neues Flag
@return keiner
*/
void vokabel::tag_setzen(int nflag)
{
  this->tag = nflag;
}

/** Setzt die Prozent der Richtigkeit der Antwort auf den "ubergebenen Wert.
@param perc Neue Prozentzahl
@return keiner
*/
void vokabel::prozent_setzen(double perc)
{
  this->prozent = perc;
}



/** Setzt den Vorher-Zeiger auf den "ubergebenen Wert.
@param zeiger Neuer Zeiger
@return keiner
*/
void vokabel::vorher_setzen(class vokabel *zeiger)
{
  this->vorher = zeiger;
}

/** Setzt den Naechster-Zeiger auf den "ubergebenen Wert.
@param zeiger Neuer Zeiger
@return keiner
*/
void vokabel::naechster_setzen(class vokabel *zeiger)
{
  this->naechster = zeiger;
}



/* Zurckliefern von Daten */

/** Liefert die Muttersprachen-Vokabel zur"uck.
@param str Neuer String
@return keiner
*/
void vokabel::nv_holen(char *str)
{
  this->nv->holen(str);
}

/** Liefert die Muttersprachen-Bemerkung zur"uck.
@param str Neuer String
@return keiner
*/
void vokabel::nr_holen(char *str)
{
  this->nr->holen(str);
}

/** Liefert die Fremdsprachen-Vokabel zur"uck.
@param str Neuer String
@return keiner
*/
void vokabel::fv_holen(char *str)
{
  this->fv->holen(str);
}

/** Liefert die Fremdsprachen-Bemerkung zur"uck.
@param str Neuer String
@return keiner
*/
void vokabel::fr_holen(char *str)
{
  this->fr->holen(str);
}

/** Liefert die Flag-Variable (gepr"uft/nicht gepr"uft) zur"uck.
@return Flag-Variable
*/
int vokabel::tag_holen(void)
{
  return(this->tag);
}

/** Liefert die Prozent der Richtigkeit der Antwort zur"uck.
@return Prozent der Richtigkeit
*/
double vokabel::prozent_holen(void)
{
  return(this->prozent);
}

/** Liefert den Vorher-Zeiger zur"uck.
@return Vorher-Zeiger
*/
class vokabel * vokabel::vorher_holen(void)
{
  return(this->vorher);
}

/** Liefert den Naechster-Zeiger zur"uck.
@return Naechster-Zeiger
*/
class vokabel * vokabel::naechster_holen(void)
{
  return(this->naechster);
}

/********************** Klasse vokabel_l ***********************/

/** Konstruktor der Klasse "'vokabel_l"'.
@return keiner
*/
vokabel_l::vokabel_l()
{
  /* Initialisieren der Variablen */
  this->anfang = NULL;
  this->ende = NULL;
  this->aktuell = NULL;

  this->nvstyle = GERMAN;
  this->nrstyle = GERMAN;
  this->fvstyle = RUSSIAN;
  this->frstyle = RUSSIAN;
  this->emphdiff = EMPHDIFF;
  this->killemph = 1;
  this->killbrack = 1;
  this->nvshow = 1;
  this->nrshow = 1;
  this->fvshow = 0;
  this->frshow = 0;
  this->askboth = 0;
  this->filename[0] = 0;
  this->boxno = 0;
  this->native = 0;

}

/** Destruktor der Klasse "'vokabel_l"'.
@return keiner
*/
vokabel_l::~vokabel_l()
{
  this->alle_loeschen();
}

/** F"ugt eine neue Vokabel in die Liste ein.
@return keiner
*/
void vokabel_l::einfuegen(void)
{
  vokabel *neu, *tmp;

  neu = (class vokabel *) new vokabel;

  if (neu != NULL)
  {
    /* Ist "uberhaupt schon ein Element eingef"ugt? */
    if (this->anfang == NULL)
    {
      /* Nein */
      neu->vorher_setzen(NULL);
      neu->naechster_setzen(NULL);
      this->aktuell = neu;
      this->anfang = neu;
      this->ende = neu;
    }
    else
    {
      /* Einfuegen am Anfang der Liste? */
      if (this->aktuell != this->ende)
      {
	/* Ja, wir sind mitten in der Liste */
	neu->vorher_setzen(this->aktuell);
	tmp = this->aktuell->naechster_holen();
	tmp->vorher_setzen(neu);
	neu->naechster_setzen(tmp);
	this->aktuell->naechster_setzen(neu);
	this->aktuell = neu;
      }
      else
      {
	/* Nein, wir haengen den Eintrag an */
	neu->vorher_setzen(this->aktuell);
	neu->naechster_setzen(NULL);
	this->aktuell->naechster_setzen(neu);
	this->aktuell = neu;
	this->ende = neu;
      }
    }
  }
  else
  {
    /* Fehlermeldung */
    DisplayErrorOccured();
    printf("vokabel_l::einfuegen : Konnte keinen Speicher fr new allozieren!\n");
    DisplayFreeMemoryMessage();
  }

}

/** L"oscht die aktuelle Person aus der Liste.
@return keiner
*/
void vokabel_l::loeschen(void)
{
  vokabel *tmp1, *tmp2;

  /* Gibt es "uberhaupt etwas zu l"oschen? */
  if (this->anfang != NULL)
  {
    /* Ja, wir k"onnen den aktuellen Eintrag l"oschen. */

    /* Wo befinden wir uns in der Liste? */
    if (this->aktuell == this->anfang)
    {
      /* Wir sind am Anfang der Liste */

      /* Ist dies der einzige Eintrag in der Liste? */
      if (this->aktuell == this->ende)
      {
	/* Ja, es ist der einzige Eintrag. */
	delete this->aktuell;
	this->aktuell = NULL;
	this->anfang = NULL;
	this->ende = NULL;
      }
      else
      {
	/* Nein, es sind noch mehr als ein Eintrag vorhanden. */

	this->anfang = this->aktuell->naechster_holen();
	this->anfang->vorher_setzen(NULL);
	delete this->aktuell;
	this->aktuell = this->anfang;
      }
    }
    else
    {
      if (this->aktuell == this->ende)
      {
	/* Wir sind am Ende der Liste, es sind mehr als */
	/* ein Eintrag vorhanden. */
	tmp1 = this->aktuell->vorher_holen();
	delete this->aktuell;
	this->aktuell = tmp1;
	this->ende = tmp1;
	this->aktuell->naechster_setzen(NULL);
      }
      else
      {
	/* Wir sind mitten in der Liste */
	tmp1 = this->aktuell->vorher_holen();
	tmp2 = this->aktuell->naechster_holen();
	tmp1->naechster_setzen(tmp2);
	tmp2->vorher_setzen(tmp1);
	delete this->aktuell;
	this->aktuell = tmp2;
      }
    }
  }
}

/** L"oscht alle Aktionen aus der Liste.
@return keiner
*/
void vokabel_l::alle_loeschen(void)
{

  while (this->anfang != NULL)
  {
    this->loeschen();
  }
}


/** Bewegt den Zeiger in der Liste einen Schritt vor, falls m"oglich.
@return keiner
*/
void vokabel_l::vor(void)
{
  if ((this->aktuell != NULL) && (this->aktuell->naechster_holen() != NULL))
  {
    this->aktuell = this->aktuell->naechster_holen();
  }
}

/** Bewegt den Zeiger in der Liste einen Schritt zur"uck, falls m"oglich.
@return keiner
*/
void vokabel_l::zurueck(void)
{
  if ((this->aktuell != NULL) && (this->aktuell->vorher_holen() != NULL))
  {
    this->aktuell = this->aktuell->vorher_holen();
  }
}

/** Bewegt den Zeiger in der Liste zehn Schritte vor, falls m"oglich.
@return keiner
*/
void vokabel_l::zehn_vor(void)
{
  int i;

  for (i = 0; i < 10; i++)
  {
    this->vor();
  }

}

/** Bewegt den Zeiger in der Liste zehn Schritte zur"uck, falls m"oglich.
@return keiner
*/
void vokabel_l::zehn_zurueck(void)
{
  int i;

  for (i = 0; i < 10; i++)
  {
    this->zurueck();
  }

}

/** Bewegt den Zeiger an den Anfang der Liste.
@return keiner
*/
void vokabel_l::zum_anfang(void)
{
  if (this->aktuell != NULL)
  {
    this->aktuell = this->anfang;
  }
}

/** Bewegt den Zeiger an das Ende der Liste.
@return keiner
*/
void vokabel_l::zum_ende(void)
{
  if (this->aktuell != NULL)
  {
    this->aktuell = this->ende;
  }
}

/** Bewegt den Zeiger auf den Eintrag mit der "ubergebenen Nummer.
@param idx Nummer
@return keiner
*/
void vokabel_l::setze_aktuell_auf_nr(long int idx)
{
  long int cnt = 1;

  this->aktuell = this->anfang;

  while ((this->aktuell != this->ende) && (cnt < idx))
  {
    this->aktuell = this->aktuell->naechster_holen();
    cnt++;
  }
}

/* Setzen von Daten */

/** Setzt den neuen Dateinamen.
@param str Neuer Name
@return keiner
*/
void vokabel_l::filename_setzen(char *str)
{
  strcpy(this->filename, str);
}

/* Zurckliefern von Daten */

/** Liefert den Wert der entsprechenden Variablen zur"uck.
@param str Zeiger auf den neuen String
@return Wert
*/
void vokabel_l::filename_holen(char *str)
{
  strcpy(str, this->filename);
}

/** Ermittelt die Anzahl der Vokabeln.
@return Anzahl
*/
int vokabel_l::ermittle_anzahl(void)
{
  int cnt = 0;

  if (this->anfang != NULL)
  {
    this->aktuell = this->anfang;

    while (this->aktuell != this->ende)
    {
      this->vor();
      cnt++;
    }
    cnt++;
  }

  return(cnt);
}

/** Ermittelt den Gesamtdurchschnitt der Prozente
@return Durchschnitt
*/
double vokabel_l::ermittle_prozent(void)
{
  int cnt = 0;
  double res = 0.0;

  if (this->anfang != NULL)
  {
    this->aktuell = this->anfang;

    while (this->aktuell != this->ende)
    {
      if (this->aktuell->tag_holen() == 1)
      {
	res += this->aktuell->prozent_holen();
	cnt++;
      }

      this->vor();
    }

    if (this->aktuell->tag_holen() == 1)
    {
      res += this->aktuell->prozent_holen();
      cnt++;
    }

  }

  return((double) (res/(cnt*1.0)));

}

/** Ermittelt die Anzahl der richtigen Vokabeln.
@return Anzahl
*/
int vokabel_l::ermittle_richtig(void)
{
  int cnt = 0;

  if (this->anfang != NULL)
  {
    this->aktuell = this->anfang;

    while (this->aktuell != this->ende)
    {
      if ((this->aktuell->tag_holen() == 1) && (this->aktuell->prozent_holen() == 100.0))
      {
	cnt++;
      }

      this->vor();
    }

    if ((this->aktuell->tag_holen() == 1) && (this->aktuell->prozent_holen() == 100.0))
    {
      cnt++;
    }

  }

  return(cnt);

}

/** Setzt alle Tags und Prozentzahlen zurueck.
@return keiner
*/
void vokabel_l::initialisieren(void)
{

  if (this->anfang != NULL)
  {
    this->aktuell = this->anfang;

    while (this->aktuell != this->ende)
    {
      this->aktuell->tag_setzen(0);
      this->aktuell->prozent_setzen(0.0);

      this->vor();
    }

    this->aktuell->tag_setzen(0);
    this->aktuell->prozent_setzen(0.0);

  }

}



/** Zeigt den Datensatz der aktuellen Vokabel an.
@return keiner
*/
void vokabel_l::zeigen(void)
{
  char dummy[MAXSTR];

  if (this->aktuell != NULL)
  {

    if (native == 0)
    {
      if (this->nvshow==1)
      {
	this->aktuell->nv_holen(dummy);
	printf("NV: %s\n", dummy);
      }
      if (this->nrshow==1)
      {
	this->aktuell->nr_holen(dummy);
	printf("NR: %s\n", dummy);
      }
      if (this->fvshow==1)
      {
	this->aktuell->fv_holen(dummy);
	printf("FV: %s\n", dummy);
      }
      if (this->frshow==1)
      {
	this->aktuell->fr_holen(dummy);
	printf("FR: %s\n", dummy);
      }
    }
    else
    {
      if (this->fvshow==1)
      {
	this->aktuell->nv_holen(dummy);
	printf("NV: %s\n", dummy);
      }
      if (this->frshow==1)
      {
	this->aktuell->nr_holen(dummy);
	printf("NR: %s\n", dummy);
      }
      if (this->nvshow==1)
      {
	this->aktuell->fv_holen(dummy);
	printf("FV: %s\n", dummy);
      }
      if (this->nrshow==1)
      {
	this->aktuell->fr_holen(dummy);
	printf("FR: %s\n", dummy);
      }
    }


  }
  else
  {
    printf("Es sind keine Vokabeln vorhanden!\n");
    if (boxno == 0)
    {
      printf("Bitte wechseln Sie in den Editiermodus mit F2!\n");
    }
    else
    {
      printf("Bitte fgen Sie eine Vokabel mit der Einfg-Taste ein!\n");
    }
  }
}

/** Zeigt den kompletten Datensatz der aktuellen Vokabel an.
@return keiner
*/
void vokabel_l::alles_zeigen(void)
{
  char dummy[MAXSTR];

  if (this->aktuell != NULL)
  {

	this->aktuell->nv_holen(dummy);
	printf("NV: %s\n", dummy);
	this->aktuell->nr_holen(dummy);
	printf("NR: %s\n", dummy);
	this->aktuell->fv_holen(dummy);
	printf("FV: %s\n", dummy);
	this->aktuell->fr_holen(dummy);
	printf("FR: %s\n", dummy);

  }
  else
  {
    printf("Es sind keine Vokabeln vorhanden!\n");
    if (boxno == 0)
    {
      printf("Bitte wechseln Sie in den Editiermodus mit F2!\n");
    }
    else
    {
      printf("Bitte fgen Sie eine Vokabel mit der Einfg-Taste ein!\n");
    }
  }
}

/** Testet die Vokabeln
@return keiner
*/
void vokabel_l::test(void)
{
  int ende=0,tcnt=0,vocnum=0,voccnt=this->ermittle_anzahl();
  char dummy[MAXSTR],ans1[MAXSTR],ans2[MAXSTR];
  double perc1=0.0,perc2=0.0;

  /* Sind ueberhaupt Vokabeln vorhanden? */
  if (this->anfang == NULL)
  {
    return;
  }

  /* Alle Vokabeln initialisieren  */
  this->initialisieren();

  while ((tcnt<voccnt) && (ende==0))
  {
    tcnt++;

    vocnum=tcnt;

    this->setze_aktuell_auf_nr(vocnum);

    while (aktuell->tag_holen() != 0)
    {
      if (this->aktuell == this->ende) this->aktuell = this->anfang;
      else this->vor();

    }

    this->zeigen();

    if (this->askboth==1)
    {
      ans1[0]=0;
      ans2[0]=0;
      /* Beide Vokabeln berprfen */
      if (this->native==1)
      {
	printf("ans1:\n");
	scanf("%s",ans1);
	this->aktuell->fv_holen(dummy);
	perc1=(double) CheckAnswer(ans1,dummy, this->killbrack, this->killemph);
	printf("ans2:\n");
	scanf("%s",ans2);
	this->aktuell->fr_holen(dummy);
	perc2=(double) CheckAnswer(ans2,dummy, this->killbrack, this->killemph);

	this->aktuell->fv_holen(dummy);
	printf("Richtig ans1 = %s:\n", dummy);
	this->aktuell->fr_holen(dummy);
	printf("Richtig ans2 = %s:\n", dummy);


	/* Prozentzahlen einblenden */
	printf("Prozent 1 %5.2f",perc1);
	printf("Prozent 2 %5.2f",perc2);

      }
      else
      {
	printf("ans1:\n");
	scanf("%s",ans1);
	printf("ans2:\n");
	scanf("%s",ans2);
	this->aktuell->nv_holen(dummy);
	perc1=(double) CheckAnswer(ans1,dummy, this->killbrack, this->killemph);
	this->aktuell->nr_holen(dummy);
	perc2=(double) CheckAnswer(ans2,dummy, this->killbrack, this->killemph);


	this->aktuell->nv_holen(dummy);
	printf("Richtig ans1 = %s:\n", dummy);
	this->aktuell->nr_holen(dummy);
	printf("Richtig ans2 = %s:\n", dummy);


	/* Prozentzahlen einblenden */
	printf("Prozent 1 %5.2f",perc1);
	printf("Prozent 2 %5.2f",perc2);

      }
      perc1=(double) (perc1+perc2)/2.0;

    }
    else
    {
      ans1[0]=0;
      /* Nur erste Vokabel berprfen */
      if (native==1)
      {
	printf("ans:\n");
	scanf("%s",ans1);
	this->aktuell->fv_holen(dummy);
	perc1=(double) CheckAnswer(ans1,dummy, this->killbrack, this->killemph);

	/* Richtige Antwort anzeigen */
	printf("Richtig ans = %s:\n", dummy);

	/* Prozentzahlen einblenden */
	printf("Prozent %5.2f",perc1);

      }
      else
      {
	printf("ans:\n");
	scanf("%s",ans1);
	this->aktuell->nv_holen(dummy);
	perc1=(double) CheckAnswer(ans1,dummy, this->killbrack, this->killemph);

	/* Richtige Antwort anzeigen */
	printf("Richtig ans = %s:\n", dummy);

	/* Prozentzahlen einblenden */
	printf("Prozent %5.2f",perc1);

      }
    }

    this->aktuell->prozent_setzen(perc1);
    this->aktuell->tag_setzen(1);

    printf("Weiter? (q = ende)\n");
    scanf("%s", dummy);

    if (dummy[0] == 113) ende = 1;

  }

  perc1 = this->ermittle_prozent();

  printf("Getestete Vokabeln: %d\n", tcnt);

  printf("Absolut richtig: %d\n", this->ermittle_richtig());

  printf("Durchschnittliche Richtigkeit: %5.2f\n", perc1);

  printf("RETURN druecken\n");
  scanf("%s", dummy);

  this->initialisieren();

}





/** Zeigt die Hilfe fuer das Editieren der Vokabeln an.
@return keiner
*/
void vokabel_l::editieren_hilfe(void)
{
  printf("*** Hilfe fr das \"Editieren\" ***\n");
  printf("\nSie knnen folgende Aktionen auslsen:\n\n");
  printf("F1  - Dieser Hilfebildschirm\n");
  printf("F2  - Editieren an/aus\n");
  printf("F5  - Lschen aller Vokabeln\n");
  printf("F10 - Beenden des Editierens\n");

  printf("\nCursor-Links  - Eine Vokabel zurck\n");
  printf("Cursor-Rechts - Eine Vokabel vor\n");
  printf("Cursor-Hoch   - Einen Textrahmen zurck\n");
  printf("Cursor-Runter - Einen Textrahmen vor\n");
  printf("TAB           - Einen Textrahmen vor\n");
  printf("Bild-Hoch     - 10 Vokabeln zurck\n");
  printf("Bild-Runter   - 10 Vokabeln vor\n");
  printf("Pos1          - Zur ersten Vokabel\n");
  printf("Ende          - Zur letzten Vokabel\n");
  printf("Einfg         - Neue Vokabel einfgen\n");
  printf("Entf          - Aktuelle Vokabel lschen\n");

  printf("\nRETURN - Text in den aktuellen Textrahmen eingeben\n");
  printf("         (Hierzu mu der Rahmen rot sein = Editieren an!)\n");

  printf("\n<< Bitte drcken Sie eine Taste >>\n");
}

/** Pr"uft den "ubergebenen String.
@param str String
@param nummer Nummer der Box
@return Wert des Vergleiches
*/
int vokabel_l::pruefe(char *str, int nummer)
{

  return(0);

}

/** Pr"uft auf doppelte Vorkommen des "ubergebenen Strings.
@param str String
@param nummer Nummer der Box
@return keiner
*/
void vokabel_l::pruefe_auf_doppelt(char *str, int nummer)
{
  class vokabel *tmp = this->aktuell;
  int end=0;

  this->aktuell = this->anfang;

  if ((str[0]!=0) && (this->anfang != NULL))
  {
    while ((this->aktuell != this->ende) && (end==0))
    {
      end = this->pruefe(str, nummer);
      this->vor();
    }

    end = this->pruefe(str, nummer);
  }

  this->aktuell = tmp;
}

/** Editiert die entsprechende Vokabel-Box.
@param nummer Nummer der Box
@return keiner
*/
void vokabel_l::editiere_box(int nummer)
{
  char newstr[MAXSTR], hold[MAXSTR];

  switch (nummer)
  {
    case 1:this->aktuell->nv_holen(newstr);
	   strcpy(hold,newstr);
	   printf("Bitte neue MV eingeben:\n");
	   scanf("%s", newstr);
	   if (mystrcmp(hold, newstr)!=1)
	   {
	     this->pruefe_auf_doppelt(newstr, nummer);
	   }
	   this->aktuell->nv_setzen(newstr);
	   break;
    case 2:this->aktuell->nr_holen(newstr);
	   strcpy(hold,newstr);
	   printf("Bitte neue MB eingeben:\n");
	   scanf("%s", newstr);
	   if (mystrcmp(hold, newstr)!=1)
	   {
	     this->pruefe_auf_doppelt(newstr, nummer);
	   }
	   this->aktuell->nr_setzen(newstr);
	   break;
    case 3:this->aktuell->fv_holen(newstr);
	   strcpy(hold,newstr);
	   printf("Bitte neue FV eingeben:\n");
	   scanf("%s", newstr);
	   if (mystrcmp(hold, newstr)!=1)
	   {
	     this->pruefe_auf_doppelt(newstr, nummer);
	   }
	   this->aktuell->fv_setzen(newstr);
	   break;
    case 4:this->aktuell->fr_holen(newstr);
	   strcpy(hold,newstr);
	   printf("Bitte neue FB eingeben:\n");
	   scanf("%s", newstr);
	   if (mystrcmp(hold, newstr)!=1)
	   {
	     this->pruefe_auf_doppelt(newstr, nummer);
	   }
	   this->aktuell->fr_setzen(newstr);
	   break;
  }


}

/** Editieren der einzelnen Vokabeln in der Liste.
@return keiner
*/
void vokabel_l::editieren(void)
{
  char b[20];
  int ende=0;

  this->boxno = 0;

  while (ende==0)
  {
    this->alles_zeigen();
    printf("Boxno: %d\n", boxno);

    printf("Ihre Eingabe: ");
    scanf("%s", b);

    switch (b[0])
    {

	       /* F2 */
	       case 102:if (this->boxno != 0)
		       {
			 this->boxno = 0;
		       }
		       else
		       {
			 this->boxno = 1;
		       }
		       break;
	       /* F5 */
	       case 120:this->alle_loeschen();
		       break;
	       /* F10 */
	       case 113:ende=1;
		       break;
	       /* Pos1 */
	       case 97:this->zum_anfang();
		       break;
	       /* Up */
	       case 104:if (this->boxno != 0)
		       {
			 if (this->boxno == 1) this->boxno = 4;
			 else this->boxno = this->boxno - 1;
		       }
		       break;
	       /* Left */
	       case 108:this->zurueck();
		       break;
	       /* Right */
	       case 107:this->vor();
		       break;
	       /* Ende */
	       case 101:this->zum_ende();
		       break;
	       /* Down */
	       case 106:if (this->boxno != 0)
		       {
			 if (this->boxno == 4) this->boxno = 1;
			 else this->boxno = this->boxno + 1;
		       }
		       break;
	       /* Insert */
	       case 105:if (this->boxno != 0)
		       {
			 this->einfuegen();
		       }
		       break;
	       /* Delete */
	       case 117:if (this->boxno != 0)
		       {
			 this->loeschen();
		       }
		       break;
      /* Return */
      case 114:if (this->boxno != 0)
	      {
		this->editiere_box(this->boxno);
	      }
	      break;
    }
  }

}

/** Laden einer Liste von Vokabeln aus einer vorher gespeicherten Datei.
@return keiner
*/
void vokabel_l::laden(char *fname)
{
  char natvoc[MAXSTR],natrem[MAXSTR],forvoc[MAXSTR],forrem[MAXSTR];
  FILE *fstream;

  if (fname[0]!=0)
  {

    fstream=fopen(fname,"rb");

    if (fstream==NULL)
    {
      DisplayErrorOccured();
      printf("vokabel_l::laden : Konnte Datei \"%s\" nicht ffnen!\n", fname);
      DisplayWrongFileMessage();
      return;
    }

    /* Liste lschen */
    this->alle_loeschen();

    /* Optionen lesen */
    fscanf(fstream,"%d %d %d %d %d %d %d %d %d\n",&(this->nvstyle),
	   &(this->nrstyle),&(this->fvstyle),&(this->frstyle),
	   &(this->nrshow),&(this->frshow),&(this->askboth),
	   &(this->killemph),&(this->killbrack));

    /* Erste Vokabel lesen */
    fgets(natvoc,MAXSTR,fstream);
    fgets(natrem,MAXSTR,fstream);
    fgets(forvoc,MAXSTR,fstream);
    fgets(forrem,MAXSTR,fstream);

    UnCr(natvoc);
    UnCr(natrem);
    UnCr(forvoc);
    UnCr(forrem);

    this->einfuegen();
    this->aktuell->nv_setzen(natvoc);
    this->aktuell->nr_setzen(natrem);
    this->aktuell->fv_setzen(forvoc);
    this->aktuell->fr_setzen(forrem);

    fgets(natvoc,MAXSTR,fstream);
    UnCr(natvoc);

    while (feof(fstream)==0)
    {

      fgets(natrem,MAXSTR,fstream);
      fgets(forvoc,MAXSTR,fstream);
      fgets(forrem,MAXSTR,fstream);

      UnCr(natrem);
      UnCr(forvoc);
      UnCr(forrem);

      /* Neue Vokabel mit Werten belegen */

      this->einfuegen();
      this->aktuell->nv_setzen(natvoc);
      this->aktuell->nr_setzen(natrem);
      this->aktuell->fv_setzen(forvoc);
      this->aktuell->fr_setzen(forrem);


      fgets(natvoc,MAXSTR,fstream);
      UnCr(natvoc);
    }

    fclose(fstream);
  }

}

/** Speichern einer Vokabelliste in eine Datei.
@return keiner
*/
void vokabel_l::speichern(char *fname)
{
  char dummy[MAXSTR];
  FILE *fstream;

  if ((fname[0]!=0) && (this->anfang != NULL))
  {
    fstream=fopen(fname,"wb");

    if (fstream==NULL)
    {
      DisplayErrorOccured();
      printf("SaveVocs: Konnte Datei \"%s\" nicht ffnen!\n", fname);
      DisplayWrongFileMessage();
      return;
    }

    /* An den Anfang springen */
    this->zum_anfang();

    /* Optionen sichern */
    fprintf(fstream,"%d %d %d %d %d %d %d %d %d\n",this->nvstyle,
	    this->nrstyle,this->fvstyle,this->frstyle,this->nrshow,
	    this->frshow,this->askboth,this->killemph,this->killbrack);

    while (this->aktuell != this->ende)
    {
      this->aktuell->nv_holen(dummy);
      fprintf(fstream,"%s\n",dummy);
      this->aktuell->nr_holen(dummy);
      fprintf(fstream,"%s\n",dummy);
      this->aktuell->fv_holen(dummy);
      fprintf(fstream,"%s\n",dummy);
      this->aktuell->fr_holen(dummy);
      fprintf(fstream,"%s\n",dummy);

      this->vor();
    }

    this->aktuell->nv_holen(dummy);
    fprintf(fstream,"%s\n",dummy);
    this->aktuell->nr_holen(dummy);
    fprintf(fstream,"%s\n",dummy);
    this->aktuell->fv_holen(dummy);
    fprintf(fstream,"%s\n",dummy);
    this->aktuell->fr_holen(dummy);
    fprintf(fstream,"%s\n",dummy);

    fclose(fstream);
  }

}

/** Zeigt das Optionen-Men"u an.
@return keiner
*/
void vokabel_l::optionen_zeigen(void)
{
 ;

}

/** Men"u f"ur das Ver"andern der Optionen.
@return keiner
*/
void vokabel_l::optionen(void)
{
  ;

}


/** Men"u f"ur das Ausw"ahlen der einzelnen Funktionen
@return keiner
*/
void vokabel_l::menu(void)
{
  char b[20];
  int vnum=0;

  while (b[0]!=27)
  {
    printf("(1) Muttersprache testen\n");
    printf("(2) Fremdsprache testen\n");
    printf("(3) Vokabeln ndern\n");
    printf("(4) Vokabeln laden\n");
    printf("(5) Vokabeln speichern\n");
    printf("(6) Dateinamen ndern\n");
    printf("(7) Optionen ndern\n");
    printf("(ESC) Ende\n");

    vnum=this->ermittle_anzahl();
    this->zum_anfang();
    printf("Geladene Vokabeln: %d",vnum);

    printf("Ihre Eingabe: ");
    scanf("%s", b);

    switch (b[0])
    {
      case 49:this->native = 0;
	      this->test();
	      break;
      case 50:this->native = 1;
	      this->test();
	      break;
      case 51:this->editieren();
	      break;
      case 52:this->laden(this->filename);
	      break;
      case 53:this->speichern(this->filename);
	      break;
      case 54:EnterFileName(this->filename);
	      break;
      case 55:this->optionen();
	      break;

    }
  }

}

/* Ende von vokabel.cpp */

