/* Name:		vokabel.h */
/* Autor:		Dirk Bchle */
/* Datum:		11.09.2000 */
/* Zweck:		Definition der Klassen "vokabel" und "string" */

/** Definition der Klassen "'vokabel"' und "'string"'.
\file vokabel.h
\author Dirk B"achle
\version 1.3
*/


/* ----------------------------------------------- Define-Variablen */

#ifndef _vokabel_h
#define _vokabel_h

#define STRINGPIECELEN			20

/* ------------------------------------------- Klassen-Definitionen */

/** Die Klasse "'stringpiece"'. */
class stringpiece
{
  /** Teilstueck eines Strings */
  char *spiece;
  /** Zeiger auf das vorherige St"uck */
  class stringpiece *vorher;
  /** Zeiger auf das n"achste St"uck */
  class stringpiece *naechster;

public:
  /* public-Funktionen */

  /* Konstruktor */
  stringpiece(void);
  /* Destruktor */
  ~stringpiece(void);

  /* Setzen von Daten */
  void setzen(char *);
  void vorher_setzen(class stringpiece *);
  void naechster_setzen(class stringpiece *);

  /* Zurckliefern von Daten */
  void holen(char *);
  class stringpiece * vorher_holen(void);
  class stringpiece * naechster_holen(void);

};

/** Klasse der Liste "'string"' bestehend aus einzelnen
"'stringpiece"' St"ucken. */
class string
{
  /* private-Variablen */
  /** Zeiger auf das erste St"uck der Liste */
  class stringpiece *anfang;
  /** Zeiger auf das letzte St"uck der Liste */
  class stringpiece *ende;

public:

  /** Zeiger auf das aktuelle St"uck der Liste */
  class stringpiece *aktuell;

  /* public-Funktionen */

  /* Konstruktor */
  string();
  /* Destruktor */
  ~string();

  /* Einf"ugen und L"oschen */
  void einfuegen(void);
  void loeschen(void);
  void alle_loeschen(void);

  /* Bewegen */
  void vor(void);
  void zurueck(void);
  void zum_anfang(void);
  void zum_ende(void);

  /* Setzen */
  void setzen(char *);

  /* Holen */
  void holen(char *);

};

/** Die Klasse "'vokabel"'. */
class vokabel
{
  /* private-Variablen */

  /** Zeiger auf die Muttersprache-Vokabel */
  class string *nv;
  /** Zeiger auf die Muttersprache-Bemerkung */
  class string *nr;
  /** Zeiger auf die Fremdsprache-Vokabel */
  class string *fv;
  /** Zeiger auf die Fremdsprache-Bemerkung */
  class string *fr;
  /** Tag f"ur das Abfragen. Gesetzt falls Vokabel abgefragt wurde */
  int tag;
  /** Prozent der Richtigkeit der Antwort */
  double prozent;
  /** Zeiger auf die vorherige Vokabel */
  class vokabel *vorher;
  /** Zeiger auf die n"achste Vokabel */
  class vokabel *naechster;

public:
  /* public-Funktionen */

  /* Konstruktor */
  vokabel(void);
  /* Destruktor */
  ~vokabel(void);

  /* Setzen von Daten */
  void nv_setzen(char *);
  void nr_setzen(char *);
  void fv_setzen(char *);
  void fr_setzen(char *);
  void tag_setzen(int);
  void prozent_setzen(double);
  void vorher_setzen(class vokabel *);
  void naechster_setzen(class vokabel *);

  /* Zurckliefern von Daten */
  void nv_holen(char *);
  void nr_holen(char *);
  void fv_holen(char *);
  void fr_holen(char *);
  int tag_holen(void);
  double prozent_holen(void);
  class vokabel * vorher_holen(void);
  class vokabel * naechster_holen(void);

};

/** Klasse der Liste "'vokabel_l"' aller Vokabeleintr"age. */
class vokabel_l
{
  /* private-Variablen */
  /** Darstellung der Muttersprache-Vokabel */
  int nvstyle;
  /** Darstellung der Muttersprache-Bemerkung */
  int nrstyle;
  /** Darstellung der Fremdsprache-Vokabel */
  int fvstyle;
  /** Darstellung der Fremdsprache-Bemerkung */
  int frstyle;
  /** Differenz der Farben zwischen betonten und unbetonten Zeichen */
  int emphdiff;
  /** Sollen die Betonung mit "uberpr"uft werden? */
  int killemph;
  /** Sollen die Klammern vor der "uberpr"ufung entfernt werden? */
  int killbrack;
  /** Wird die Muttersprache-Vokabel vor Eingabe der Antwort gezeigt? */
  int nvshow;
  /** Wird die Muttersprache-Bemerkung vor Eingabe der Antwort gezeigt? */
  int nrshow;
  /** Wird die Fremdsprache-Vokabel vor Eingabe der Antwort gezeigt? */
  int fvshow;
  /** Wird die Fremdsprache-Bemerkung vor Eingabe der Antwort gezeigt? */
  int frshow;
  /** Werden sowohl Vokabel als auch Bemerkung abgefragt? */
  int askboth;
  /** Name der aktuellen Vokabel-Datei */
  char filename[FNAME_MAX];

  /** Nummer der zu editierenden Box */
  int boxno;
  /** Abfrage Fremdsprache oder Muttersprache? */
  int native;

  /** Zeiger auf die erste Vokabel der Liste */
  class vokabel *anfang;
  /** Zeiger auf die letzte Vokabel der Liste */
  class vokabel *ende;

public:

  /** Zeiger auf die aktuelle Vokabel der Liste */
  class vokabel *aktuell;

  /* public-Funktionen */

  /* Konstruktor */
  vokabel_l();
  /* Destruktor */
  ~vokabel_l();

  /* Einf"ugen und L"oschen */
  void einfuegen(void);
  void loeschen(void);
  void alle_loeschen(void);

  /* Bewegen */
  void vor(void);
  void zurueck(void);
  void zehn_vor(void);
  void zehn_zurueck(void);
  void zum_anfang(void);
  void zum_ende(void);
  void setze_aktuell_auf_nr(long int);

  /* Daten setzen */
  void filename_setzen(char *);

  /* Daten holen */
  void filename_holen(char *);

  /* Anzahl der Vokabeln ermitteln */
  int ermittle_anzahl(void);

  /* Gesamt-Prozent ermitteln */
  double ermittle_prozent(void);
  /* Anzahl der richtigen Vokabeln ermitteln */
  int ermittle_richtig(void);
  /* Vokabeln initialisieren */
  void initialisieren(void);

  /* Anzeigen der aktuellen Vokabel */
  void zeigen(void);
  void alles_zeigen(void);

  /* Vokabeln testen */
  void test(void);

  /* Editieren */
  void editieren_hilfe(void);
  int pruefe(char *, int);
  void pruefe_auf_doppelt(char *, int);
  void editiere_box(int);
  void editieren(void);

  /* Speichern/Laden */
  void laden(char *);
  void speichern(char *);

  /* Menu */
  void optionen_zeigen(void);
  void optionen(void);
  void dump_all(void);
  void menu(void);

};


#endif

/* Ende von vokabel.h */


